"""
Пакет для автоматизации взаимодействия с пакетным режимом 1С.
"""
__author__ = 'Alexander_NiMo'
__maintainer__ = __author__


__email__ = 'binipox@gmail.com'
__license__ = 'MIT'
__version__ = '0.1.4'


__all__ = (
    '__author__',
    '__email__',
    '__license__',
    '__maintainer__',
    '__version__',
)